@@ -18,6 +18,22 @@ def Iff(x: AST, y: AST):
     return Ite(x, y, Not(y))
 
 
+def And(x: AST, y: AST):
+    return Ite(x, y, Bool(False))
+
+
+def Or(x: AST, y: AST):
+    return Ite(x, Bool(True), y)
+
+
+def Xor(x: AST, y: AST):
+    return Ite(x, Not(y), y)
+
+
+def Implies(x: AST, y: AST):
+    return Ite(x, y, Bool(True))
+
+
 def wrap_builtin_binop(b: uplc_ast.BuiltInFun):
     def wrapped(x: AST, y: AST):
         return Apply(BuiltIn(b), x, y)
@@ -334,6 +350,35 @@ def MapList(l: AST, m: AST = Lambda(["x"], Var("x")), empty_list=EmptyDataList()
     )
 
 
+def FindList(l: AST, key: AST, default: AST):
+    """Returns the first element in the list where key evaluates to true - otherwise returns default"""
+    return Apply(
+        Lambda(
+            ["op"],
+            RecFun(
+                Lambda(
+                    ["f", "xs"],
+                    Ite(
+                        NullList(Var("xs")),
+                        default,
+                        Ite(
+                            Apply(Var("op"), HeadList(Var("xs"))),
+                            HeadList(Var("xs")),
+                            Apply(
+                                Var("f"),
+                                Var("f"),
+                                TailList(Var("xs")),
+                            ),
+                        ),
+                    ),
+                ),
+            ),
+        ),
+        key,
+        l,
+    )
+
+
 # Data Utils
 
 
